## Introduction

caserver-proxy, also named ca-proxy, receives the requests from the clients and distributes them to one of or both 
caserver *sync* and *async* instances, that must be running on the same host or on the hosts specified in the ca-proxy 
configuration file. A simple read request is proxied to the *sync* service, which will provide a synchronous reply.
A *subscribe* request, that implies both a synchronous reply and a *monitoring* of a quantity over time (using the Server Sent
Events model), will be forwarded to *both sync and async* services. The first will reply immediately while the second 
will start monitoring the source.

Alongside ca-proxy, you need to run *two instances of caserver*. Refer to the *INSTALL.md* file on the project page:
https://gitlab.elettra.eu/puma/server/caserver/ and proceed to the installation and setup.

### postgresql dependency 

The ca-proxy relies on a simple database to store records for every *active client*.
Each client shall periodically send a *keep alive* message containing the client *id*, which is then marked as 
active on the database. ca-proxy checks for old entries and deletes them, *unsubscribing* the vanished client 
from the *caserver async*.

Please install the *postgresql database* on your system and refer to the dedicate section below for its configuration.

### caserver-lib dependency

Install or update the caserver-lib from https://gitlab.elettra.eu/puma/server/caserver-lib
Refer to the detailed *INSTALL.md* file from the project page.

### caserver-proxy installation

> git clone https://gitlab.elettra.eu/puma/server/caserver-proxy

Check PKG_CONFIG_PATH

> echo $PKG_CONFIG_PATH 

It must contain caserver-lib, cumbia-libs, tango, omniorb, zeromq and optionally nlohmann-json.

For example, if the installation of *caserver-lib* has been done according to the instructions in
the INSTALL.md file under https://gitlab.elettra.eu/puma/server/caserver-lib/-/blob/master/INSTALL.md,
we will make sure that PKG_CONFIG_PATH contains:

/home/puma/.local/caserver-lib/lib/pkgconfig

alongside the other paths mentioned.

#### Note
If nlohmann-json is not installed system wide, we can copy the *subprojects* directory from the 
*caserver-lib* source folder to make the necessary include files available to *caserver-proxy*.

### Build 

> cd caserver-proxy

> meson builddir

> cd builddir

Set the install prefix:

> meson configure -Dprefix=/home/puma/.local/caserver-proxy


Other options (check with *meson configure* the default values, they may be already good for you)

Custom *systemd* service file location (in our example we are working under the home directory):

> meson configure -Dsystemd_service_dir=/home/puma/.local/caserver-proxy/etc/systemd

Please check the file *ca-proxy.service.in* under the top level source directory. The file is used to write the final
*ca-proxy.service* systemd service.

The program needs a configuration file. One is provided in the source tree. Set the options and if you need to 
rename it, specify its new name with the following *meson configure*:

> meson configure -Dconf_file=ca-proxy.conf

Debug or release:

> meson configure -Dbuildtype=debug

Check the options with

> meson configure

Then proceed to building and installing:

> ninja 

> ninja install 

### Systemd 

Enable the service:

> sudo systemctl enable /home/puma/.local/caserver-proxy/etc/systemd/ca-proxy.service

#### Note

If you need to specify a LD_LIBRARY_PATH because one of cumbia, tango, caserver-lib dependencies are not globally visible, please run

> sudo systemctl edit ca-proxy

and fix the environment. For example:

```
[Service]
Environment="LD_LIBRARY_PATH=/home/puma/.local/caserver-lib/lib:/home/puma/.local/cumbia-libs/lib"
```

## Database configuration 

A database *configuration script* is provided in the top level source directory: *ca-register-create.sql*.
Two tables are created:

- clients 
- register 

After installing and starting the *postgresql* database, make sure the user www-data is added to the database. If not, execute:

>  sudo -u postgres createuser www-data

and run *psql*:

> sudo -u postgres psql

From within the *postgres* prompt:

> postgres=# alter user www-data with encrypted password 'cadbdb';  

Quit psql (CTRL-D).

#### NOTE

Make sure that the user and password just configured match the ones specified in the file used in the following instruction (cadb-create.sql in our example).

To create the database, execute now

> psql -U postgres -f cadb-create.sql

If this fails, please check */etc/postgresql/10/main/pg_hba.conf* to make sure the *postgres* user is trusted.

Logging into the *postgresql* *caregister* database

> psql -U postgres caregister

inspect the tables and the schema:

```
caregister=# \dt
          List of relations
 Schema |   Name   | Type  |  Owner   
--------+----------+-------+----------
 public | clients  | table | postgres
 public | register | table | postgres
(2 rows)
```

```
caregister=# \d register
                                           Table "public.register"
  Column   |            Type             | Collation | Nullable |                   Default                   
-----------+-----------------------------+-----------+----------+---------------------------------------------
 client_id | integer                     |           | not null | nextval('register_client_id_seq'::regclass)
 time      | timestamp without time zone |           | not null | 
Indexes:
    "register_pkey" PRIMARY KEY, btree (client_id)
Referenced by:
    TABLE "clients" CONSTRAINT "clients_id_fkey" FOREIGN KEY (id) REFERENCES register(client_id) ON DELETE CASCADE
    TABLE "clients" CONSTRAINT "clients_id_fkey1" FOREIGN KEY (id) REFERENCES register(client_id)
```

### Service configuration

Since ca-proxy runs alongside two instances of *caserver*, one running as *sync* and another as *async*, please make sure that 
the configuration file is correct as far as *caserver* host and ports are concerned:

> vim ca-proxy.conf

```
ca-proxy:sync-server-address=pwma-dev.elettra.eu
ca-proxy:sync-server-port=12001
ca-proxy:async-server-address=pwma-dev.elettra.eu
ca-proxy:async-server-port=12002
```

The configuration file provided contains other directives to bind the ca-proxy to a specific port and to configure database access and 
client keep alive interval. 

The configuration file is structured in a two level option hierarchy so that the same configuration file can in principle be used
by both *caserver* and *ca-proxy* instances. The two levels are separate by *:*.

#### Start the service

> sudo systemctl start ca-proxy

Check that the service is running as expected:

> sudo systemctl status ca-proxy


