/* psql  -U postgres  -f ../cadb-create.sql */

/* SEE
 * https://gitlab.elettra.eu/puma/server/ca3-db-plugin
 * README.md
*/

DROP DATABASE IF EXISTS cadb;

CREATE DATABASE cadb;

\c cadb

CREATE TABLE srvconf (
    conf_id SERIAL UNIQUE,
	srvnam varchar(255),
	addr varchar(64) NOT NULL,
	port integer NOT NULL,
	PRIMARY KEY(addr,port)
);

CREATE TABLE service (
    id SERIAL PRIMARY KEY,
	started timestamp,
	expected timestamp,
	conf_id integer REFERENCES srvconf (conf_id)
);

/* do not set PRIMARY key (source,chan) because two services
 * can monitor the same source on the same channel
 */
CREATE TABLE activity (
    srv_id integer NOT NULL,
	cli_id integer NOT NULL,
	"source" varchar(512) NOT NULL,
	chan varchar (1024) NOT NULL
);

/*
 * the following tables are used (r/w) by the supervisor
 */

CREATE TABLE supervisor (
    id SERIAL PRIMARY KEY,
	started timestamp NOT NULL,
	stopped timestamp,
	url varchar(2048) NOT NULL
);

CREATE TABLE "recovery" (
    id SERIAL PRIMARY KEY,
	srv_id integer NOT NULL,
	started timestamp NOT NULL,
	supervisor_id integer NOT NULL REFERENCES supervisor (id)
);

CREATE TABLE recover_operations (
    id SERIAL PRIMARY KEY,
	supervisor_id INTEGER NOT NULL REFERENCES supervisor (id),
	started timestamp NOT NULL,
	complete timestamp,
	success boolean
);

CREATE TABLE recover_activities (
    operation_id INTEGER NOT NULL,
	"source" varchar(2048) NOT NULL,
	"chan" varchar(2048) DEFAULT NULL,
	"cli_id" integer NOT NULL,
	from_srv_conf INTEGER NOT NULL REFERENCES srvconf (conf_id),
	to_srv_conf INTEGER DEFAULT NULL REFERENCES srvconf (conf_id)
);

CREATE TABLE register (
    client_id SERIAL PRIMARY KEY,
	time timestamp NOT NULL
);

COMMENT ON TABLE register IS 'accessed by ca-proxy (RW), caserver-supervisor-plugin (RO) and ca-supervisor (RO). Stores the client ID and the client last keep alive timestamp';


CREATE TABLE "clients" (
    id integer PRIMARY KEY REFERENCES register (client_id) ON DELETE CASCADE,
	addr varchar(64) NOT NULL,
	FOREIGN KEY(id) REFERENCES register(client_id)
);

/* get table desc and their comments with \d+ */
COMMENT ON TABLE clients IS 'accessed by ca-proxy when a new client registers';

GRANT ALL PRIVILEGES ON service TO "www-data";
GRANT ALL PRIVILEGES ON activity TO "www-data";
GRANT ALL PRIVILEGES ON srvconf TO "www-data";
GRANT ALL PRIVILEGES ON "recovery" TO "www-data";
GRANT ALL PRIVILEGES ON supervisor TO "www-data";
GRANT ALL PRIVILEGES ON recover_operations TO "www-data";
GRANT ALL PRIVILEGES ON "recover_activities" TO "www-data";
GRANT USAGE, SELECT ON ALL SEQUENCES IN SCHEMA public TO "www-data";
GRANT ALL PRIVILEGES ON clients TO "www-data";
GRANT ALL PRIVILEGES ON register TO "www-data";

GRANT USAGE, SELECT ON ALL SEQUENCES IN SCHEMA public TO "www-data";

/*
GRANT ALL PRIVILEGES ON caservers TO "giacomo";
GRANT ALL PRIVILEGES ON caservers TO "www-data";
GRANT ALL PRIVILEGES ON sources TO "giacomo";
GRANT ALL PRIVILEGES ON sources TO "www-data";
*/
