#ifndef CAREGISTRAR_H
#define CAREGISTRAR_H

#include <cuactivity.h>

class CaRegistrarPrivate;
class CuData;
class CaDbH;

class CaRegiEvent {
public:
    enum Type { RERecord, RECheckExpired, REUnsubscribe };

    CaRegiEvent(const unsigned long long& to, const std::string& ho, int _fd, bool unsub)
        : tok(to), host(ho), port(std::string()), fd(_fd), type(unsub ? REUnsubscribe : RERecord)  {}

    CaRegiEvent() : tok(0), type(RECheckExpired) {}

//    CaRegiEvent(const unsigned long long& id, const std::string& caserver_a,
//                const std::string& caserver_p, const std::string& _src, int _fd) :
//        tok(id), host(caserver_a), port(caserver_p), src(_src), fd(_fd), type(RERecord) {}

    const unsigned long long tok;
    const std::string host, port;
    const std::string src;
    int fd;
    Type type;
};

class CaRegistrar : public CuActivity
{
public:
    enum Type { CaRegistrarType = 2004 };
    CaRegistrar(const CuData &connp);
    virtual ~CaRegistrar();

    void new_event(CaRegiEvent *e) const;
    std::string& msg() const;
    bool err() const;
    int clientRecordTtl() const;

    void stop();
    void setClientRecordTtl(int sec);

    bool id_valid(const unsigned long long &id) const;

    bool db_connected() const;

private:
    CaRegistrarPrivate *d;

    unsigned long long  m_insert_record(const std::string& addr, time_t &expires);
    unsigned long long m_update_record(const long long unsigned &id, const std::string& addr, time_t &expires);
    bool m_record_up_to_date(const unsigned long int &id, CaDbH *dbh)  const;
    bool m_delete_expired_records();
    CuData m_make_ok_ttl_id(time_t &ttl, const unsigned long long& id) const;
    bool m_get_up_to_date_records(CuData &out);
    bool m_delete_unsubscribed(const unsigned long long& id);
    bool m_get_srv_id();
    // CuActivity interface
public:
    int getType() const;
    void event(CuActivityEvent *e);
    bool matches(const CuData &token) const;
    int repeat() const;

protected:
    void init();
    void execute();
    void onExit();
};

#endif // CAREGISTRAR_H
