#include "caprosock.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>
#include <unistd.h>
#include <string.h> // memset
#include <arpa/inet.h> // inet_ntop
#include <cahttpmsg.h>

#define _BUFSIZ 512

int CaProSock::sock_read(int sofd, std::string &in, std::string &msg) {
    msg.clear();
    in.clear();
    char rbuf[_BUFSIZ];
    int nread, totread = 0;
    bool complete = false;
    CaHttpMsg httpa;
    while(!complete && (nread = read(sofd, rbuf, _BUFSIZ -1)) > 0) {
        rbuf[nread] = '\0';
        complete = httpa.add(rbuf);
        totread += nread;
        if(httpa.state == CaHttpMsg::Error)
            break;
    }
    if(nread < 0)
        msg = "ca-pro-sock.m_sock_read: socket error: " + std::string(strerror(errno));
    if(!complete || httpa.state == CaHttpMsg::Error)
        msg = "ca-pro-sock.m_sock_read: invalid HTTP message: \"" + httpa.buf + "\"";
    else
        in = std::move(httpa.buf);
    return nread >= 0 ? totread : nread;
}

int CaProSock::sock_write(int sofd, const char *buf, std::string &msg) {
    msg.clear();
    int totclibw = 0, clibw = 0, wlen = strlen(buf);
    // write rbuf back to client fd
    while(totclibw < wlen && clibw >= 0) {
        clibw = send(sofd, buf + totclibw, wlen - totclibw < _BUFSIZ ? wlen - totclibw : _BUFSIZ, MSG_NOSIGNAL);
        if(clibw < 0)
            msg = "ca-pro-sock.m_sock_write: write failed : " + std::string(strerror(errno));
        totclibw += clibw;
    }
    return clibw >=0 ? totclibw : clibw;
}


int CaProSock::sock_connect(const char *addr, const char *port, std::string &msg, unsigned timeout) {
    int casfd = -1;
    int  s;
    struct addrinfo hints;
    struct addrinfo *result = nullptr, *rp = nullptr;
    msg = std::string();
    memset(&hints, 0, sizeof(struct addrinfo));
    hints.ai_family = AF_UNSPEC;    /* Allow IPv4 or IPv6 */
    hints.ai_socktype = SOCK_STREAM; /* TCP socket */
    hints.ai_flags = 0;
    hints.ai_protocol = 0;          /* Any protocol */
    hints.ai_flags |= AI_NUMERICSERV;
    s = getaddrinfo(addr, port, &hints, &result);
    if (s != 0) {
        msg = "ca-pro-sock.m_connect: getaddrinfo failed: " + std::string(gai_strerror(s));
    }
    else {
        for (rp = result; rp != NULL; rp = rp->ai_next) {
            casfd = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
            if (casfd == -1)
                continue;
            if (connect(casfd, rp->ai_addr, rp->ai_addrlen) != -1) {
                break;                  /* Success */
            }
            close(casfd);
            casfd = -1;
        }

        if(rp == nullptr) {
            msg = std::string(addr) + ":" + std::string(port) + ": " + std::string(strerror(errno));
            casfd = -1;
        }
        freeaddrinfo(result);           /* No longer needed */
        if(casfd > -1 && timeout > 0) {
            struct timeval tv { timeout, 0 };
            if(setsockopt(casfd, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv)) < 0)
                msg = "ca-pro-sock.m_connect: setsockopt failed: " + std::string(strerror(errno));
            else
                printf("setsockopt ok timeout %u\n", timeout);
        }
    }
    return casfd;
}
