#include "caproxyhttph.h"
#include "caregistrarlistener.h"
#include "caproxythreadpool.h"
#include "caproxyjson_h.h"
#include <cajson-data-encoder.h>
#include <calog.h>

class CaRegistrarListenerPrivate {
public:
    CaRegistrarListenerPrivate() : propool(nullptr), log(nullptr)  {}
    CaProxyThreadPool *propool;
    CuLogImplI *log;
    CaProxyHttpH phttph;
};

CaRegistrarListener::CaRegistrarListener() {
    d = new CaRegistrarListenerPrivate;
}

CaRegistrarListener::~CaRegistrarListener() {
    delete d;
}

void CaRegistrarListener::init(CaProxyThreadPool *pp, CuLogImplI *log) {
    d->propool = pp; d->log = log;
}

void CaRegistrarListener::onProgress(int step, int total, const CuData &data) { }

void CaRegistrarListener::onResult(const CuData &data) {
    const int cli_fd = data.i("fd");
    if(data.containsKey("srv_id")) {
        d->log->write("ca-registrar", data.s("msg"), CuLog::LevelInfo);
        d->propool->set_srv_id(data["srv_id"].toLongInt());
    }
    else if(data.b("err") && cli_fd <= 0)
        d->log->write("ca-registrar", data.s("msg"));
    else if(data.b("err") && cli_fd > 0)
        d->propool->msg_send_cli(cli_fd, CaProxyEvent::CaPHttp500, CaJsonDataEnc().toString(std::list<CuData> {data.remove("fd").set("msg", "database error: " + data.s("msg"))} ));
    else if(data.containsKey("exp") && data.containsKey("id") && cli_fd > 0)
        d->propool->msg_send_cli(cli_fd, CaProxyEvent::CaPHttpReplyOk, CaJsonDataEnc().toString(std::list<CuData> {data.remove("fd")} ));
    else if(data.containsKey("uptodate"))
        d->propool->msg_send_caserver(CaProxyRoute(CaProxyRoute::ToAsync), CaProxyEvent::CaPHttpPost, CaProxyJsonH().make_up2date(data.ULLV("uptodate")), CaProxyEvent::CaPEUp2Date);
    else if(data.containsKey("u") && cli_fd > 0) {
        pretty_pri("see if string can become string view");
        d->propool->do_proxy(cli_fd, CaProxyEvent::CaPHttpPost, std::string(), std::string(), CaProxyJsonH().make_unsubscribe(data.ULLV("u")), std::string(), 0);
    }
}

void CaRegistrarListener::onResult(const std::vector<CuData> &datalist) { }

CuData CaRegistrarListener::getToken() const {
    return CuData("type", "CaRegistrarListener");
}



